/*
 * Copyright (C) 2012 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.gig.persistence;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceException;

import org.eiichiro.jaguar.Component;
import org.eiichiro.jaguar.inject.Name;
import org.eiichiro.jaguar.lifecycle.Constructed;
import org.eiichiro.jaguar.lifecycle.Passivated;
import org.eiichiro.jaguar.scope.Singleton;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * {@code JPAEntityManagerFactoryComponent} is a representing JPA 2.0 
 * {@code EntityManagerFactory} for PostgreSQL-based Heroku database service.
 * 
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
@Name("org.eiichiro.gig.persistence.JPAEntityManagerFactory")
@Singleton
public class JPAEntityManagerFactoryComponent extends Component<EntityManagerFactory> {

	private Logger logger = LoggerFactory.getLogger(getClass());
	
	private List<String> persistenceUnits = new ArrayList<String>();
	
	private Map<String, EntityManagerFactory> factories = new ConcurrentHashMap<String, EntityManagerFactory>();
	
	/**
	 * Configures the persistence units to be connected from a 
	 * {@code PersistenceXMLProcessor}.
	 */
	@Constructed
	public void configure() {
		PersistenceXMLProcessor processor = new PersistenceXMLProcessor();
		processor.process();
		persistenceUnits = processor.persistenceUnits();
	}
	
	/** Lifecycle callback to close the {@code EntityManagerFactory} set. */
	@Passivated
	public void close() {
		for (EntityManagerFactory factory : factories.values()) {
			factory.close();
		}
		
		persistenceUnits.clear();
	}
	
	/**
	 * Returns the {@code EntityManagerFactory} instance corresponding to the 
	 * persistence unit the current application connects.
	 * If the {@code PersistenceXMLProcessor} detects multiple persistence unit 
	 * declarations, this class connects the first-detected one (Index 0 of the 
	 * persistence unit list that the {@code PersistenceXMLProcessor#persistenceUnits()} 
	 * returns).
	 * If the current application cannot access to the database service for any 
	 * reasons, this method returns <code>null</code>.
	 * 
	 * <b>Note: </b>In the future, the implementation will be improved for 
	 * multitenant applications. The persistence unit set on the current thread 
	 * via something like "Namespaces API" will be used for the 
	 * {@code EntityManagerFactory} creation.
	 * 
	 * @return The {@code EntityManagerFactory} instance corresponding to the 
	 * persistence unit the current application connects.
	 */
	@Override
	public EntityManagerFactory instance() {
		if (!persistenceUnits.isEmpty()) {
			EntityManagerFactory factory = factories.get(persistenceUnits.get(0));
			
			if (factory == null) {
				try {
					factory = Persistence.createEntityManagerFactory(persistenceUnits.get(0));
					factories.put(persistenceUnits.get(0), factory);
					logger.info("EntityManagerFactory for persistence unit ["
							+ persistenceUnits.get(0) + "] created");
				} catch (PersistenceException e) {
					logger.error("Failed to create EntityManagerFactory", e);
				}
			}
			
			return factory;
		}
		
		logger.warn("Cannot access to the database service in this application; " +
				"Returned 'EntityManagerFactory' instance is null");
		return null;
	}
	
	public List<String> persistenceUnits() {
		return persistenceUnits;
	}
	
}
